import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.lang.Exception;
import java.lang.Thread;
import PuzzleGame.*;

public class Appletris extends Applet implements Runnable
{
	public void init()
	{
		this.showStatus("Loading applet...");
		
		///Standard initiations///
		super.init();		
		super.setLayout(null);
		super.setBackground(Color.black);
		super.setForeground(Color.white);
		super.setSize(800,600);
		
		///Setup a focuslistener for the applet///
		this.focus_control = new FocusClass();
		super.addFocusListener(focus_control);
		
		///Setup a keylistener (controls) for the applet///
		super.addKeyListener( new KeyAdapter(){
					public void keyPressed(KeyEvent keye)
					{
						switch (keye.getKeyCode())
						{
							case KeyEvent.VK_LEFT:
								horisontal_move(-1);
								break;
							case KeyEvent.VK_RIGHT:
								horisontal_move(1);
								break;
							case KeyEvent.VK_UP:
								rotateRight();
								break;
							case KeyEvent.VK_DOWN:
								rotateLeft();
								break;
							case KeyEvent.VK_SPACE:
								drop();
								break;
						}
					}});
				
		///Set pieces///
		TetrisPuzzlePiece pieces[] = new TetrisPuzzlePiece[7];
		pieces[0] = new TetrisPuzzlePiece(3,"100100110");
		pieces[1] = new TetrisPuzzlePiece(3,"010010110");
		pieces[2] = new TetrisPuzzlePiece(3,"010110100");
		pieces[3] = new TetrisPuzzlePiece(3,"100110010");
		pieces[4] = new TetrisPuzzlePiece(2,"1111");
		pieces[5] = new TetrisPuzzlePiece(3,"100110100");
		pieces[6] = new TetrisPuzzlePiece(4,"1000100010001000");

		///Load piece-images///
		Image images[] = new Image[7];
		images[0] = getImage(getCodeBase(), "Images/yellow.gif");
		images[1] = getImage(getCodeBase(), "Images/blue.gif");
		images[2] = getImage(getCodeBase(), "Images/green.gif");
		images[3] = getImage(getCodeBase(), "Images/purple.gif");
		images[4] = getImage(getCodeBase(), "Images/grey.gif");
		images[5] = getImage(getCodeBase(), "Images/red.gif");
		images[6] = getImage(getCodeBase(), "Images/turquoise.gif");		

		///Load border-images///
		Image ia[] = new Image[8];
		ia[0] = getImage(getCodeBase(), "Images/Border/topborder.gif");
		ia[1] = getImage(getCodeBase(), "Images/Border/bottomborder.gif");
		ia[2] = getImage(getCodeBase(), "Images/Border/leftborder.gif");
		ia[3] = getImage(getCodeBase(), "Images/Border/rightborder.gif");
		ia[4] = getImage(getCodeBase(), "Images/Border/tlcorner.gif");
		ia[5] = getImage(getCodeBase(), "Images/Border/trcorner.gif");
		ia[6] = getImage(getCodeBase(), "Images/Border/blcorner.gif");
		ia[7] = getImage(getCodeBase(), "Images/Border/brcorner.gif");

		///Create MediaTracker and wait for images to load///
		MediaTracker temp = new MediaTracker(this);
		for (int i = 0; i < 7; i++)
		{
					temp.addImage(ia[i], 0);
					temp.addImage(images[i], 0);
		}
		temp.addImage(ia[7], 0);
		try
		{
			temp.waitForID(0);
		}
		catch(Exception exc){}
		
		///Create thread and board, init board///
		this.tetris_thread = new Thread(this, "Appletris");
		board = new TetrisGameBoard(20, 10, 20);
		board.setPieces(pieces, images);
		board.clear();
		
		///Create and set borders///
		board_frame = new FramedArea(21,11);
		nextpiece_frame = new FramedArea(5, 5);
		score_frame = new FramedArea(3, 8);
		level_frame = new FramedArea(3, 8);
		board_frame.setBorder(ia);
		nextpiece_frame.setBorder(ia);
		score_frame.setBorder(ia);
		level_frame.setBorder(ia);
		
		///Create and init pieces///
		current = board.getRandomPiece();
		current.setLocation(new Point(0,4));
		next = board.getRandomPiece();
		while (current.getImage() == next.getImage())
			next = board.getRandomPiece();
		
		this.showStatus("Finished loading applet...");
	}//init

//////////////Applet control functions///////////////////
	public void start()
	{
		this.page = this.getGraphics();
		this.tetris_thread.start();
		this.repaint();
	}//start
	
	public void stop()
	{
		this.running = false;
		this.showStatus("Applet has been stopped...");
	}//stop
	
	public void destroy()
	{
		this.board = null;
		this.tetris_thread = null;
	}//destroy

//////////////////////Graphics///////////////////////////	
	public void paint(Graphics gr)
	{
		board_frame.drawBorder(0, 0, gr);
		nextpiece_frame.drawBorder(250, 0, gr);
		score_frame.drawBorder(250, 120, gr);
		level_frame.drawBorder(250, 200, gr);
		board.drawBoard(20,20,gr);
		current.drawPiece(20,20,gr);
		
		page.clearRect(270,20,80,80);
		next.drawPiece(310-10*(next.getRightmostPosition()+1),
		60-10*(next.getBottommostPosition()+1), gr);
		
		page.drawString("By: Fist of Heaven",500,50);
		page.clearRect(500,50,150,50);
		page.drawString("Level:" + level,500,100);
		page.clearRect(500,100,150,50);
		page.drawString("Score:" + score,500,150);
	}//paint
	
	public void update(Graphics gr)
	{
		this.paint(gr);
	}

////////////////////Control functions///////////////////////
	public void rotateRight()
	{
		if (current.moves)
		{
			TetrisPuzzlePiece temppiece = current.getCopy();
			temppiece.rotateRight();
			
			while (board.checkForCollision(temppiece) == 1)
			{
				temppiece.move(0,-1);
			}
			
			if (board.checkForCollision(temppiece) == 0)
			{
				current.clearPiece(20,20,page);
				current.setLocation(temppiece.getLocation());
				current.rotateRight();
				current.drawPiece(20,20,page);
			}
			temppiece = null;
		}
	}

	public void rotateLeft()
	{
		if (current.moves)
		{
			TetrisPuzzlePiece temppiece = current.getCopy();
			///Simulate rotating left by rotating three times right///
			temppiece.rotateRight();
			temppiece.rotateRight();
			temppiece.rotateRight();
						
			while (board.checkForCollision(temppiece) == 1)
			{
				temppiece.move(0,-1);
			}
			
			if (board.checkForCollision(temppiece) == 0)
			{
				current.clearPiece(20,20,page);
				current.setLocation(temppiece.getLocation());
				current.rotateRight();
				current.rotateRight();
				current.rotateRight();
				current.drawPiece(20,20,page);
			}
			temppiece = null;
		}
	}

	public void horisontal_move(int dir)
	{
		if (current.moves)
		{
			TetrisPuzzlePiece temp = current.getCopy();
			temp.move(0, dir);
			if (board.checkForCollision(temp) == 0)
			{
				current.clearPiece(20,20,page);
				current.move(0,dir);
				current.drawPiece(20,20,page);
			}
			temp = null;
		}
	}

	public void drop()
	{
		if (current.moves)
		{
			TetrisPuzzlePiece temppiece = current.getCopy();
			while (board.checkForCollision(temppiece) == 0)
			{
				temppiece.move(1,0);
			}
			temppiece.move(-1,0);
			
			current.clearPiece(20,20,page);
			current.setLocation(temppiece.getLocation());
			current.drawPiece(20,20,page);
			current.moves = false;
		}
	}

	public void game_over()
	{
		board.clear();
	}
//////////////////////////////Game Loop/////////////////////////////////
	public void run()
	{
		this.running = true;
		int speed = 1500;
		this.score = 0;
		this.removed_rows = 0;
		this.level = 1;
		
		while (running)
		{
			while(!focus_control.active)
			{
				tetris_thread.yield();
			}
			
			try
			{
				tetris_thread.sleep(speed);
			}catch (Exception exc){}

			if (!current.moves)
			{
				board.updateBoardWith(current);
				int num_rows = board.applyGravity();

				if (num_rows > 0)
				{
					removed_rows += num_rows;

					if (removed_rows >= 10 && level < 10)
					{
						level++;
						speed -= 140;
						removed_rows = removed_rows % 10;
					}
					score += 100 * num_rows;
				}

				score += 10;
				current = next.getCopy();
				current.setLocation(new Point(0,4));
				if (board.checkForCollision(current) > 0)
					game_over();
				next = board.getRandomPiece();
				this.repaint();
			}
			else
			{
				TetrisPuzzlePiece temp = current.getCopy();
				temp.move(1,0);
				if(board.checkForCollision(temp) > 0)
				{
					current.moves = false;
				}
				else
				{
					current.clearPiece(20,20,page);
					current.move(1,0);
					current.drawPiece(20,20,page);
				}
			}
		}//while
	}//run
///////////////////////////////////////////////////////////////////////
	
	///Private class variables///
	private boolean running;
	private FocusClass focus_control;
	private Graphics page;
	private TetrisGameBoard board;
	private TetrisPuzzlePiece current, next;
	private Thread tetris_thread;
	private FramedArea nextpiece_frame,
									score_frame,
									board_frame,
									level_frame;
	private int score, level, removed_rows;
}	//class