package PuzzleGame;

/*****************************************************************************
 *  This class implements a gameboard as a matrix of numRow x numCol integers,
 and provides a number of methods for working with that gameboard. Great for
 easy checking of boundaries since puzzlegames often has simple boundaries that could be
 implemented as matrices.
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Martin Karlsson, fist_of_heaven@hotmail.com
 * @version 1.0
 */

public class GameBoard
{
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 Default constructor. Takes no parameters and sets the gameboard to a 1x1 matrix.
 */
  public GameBoard()
  {
    numCol = 1;
    numRow = 1;
    gameBoardMatrix = new int[numRow][numCol];
    clear();
  }

/*****************************************************************************
 Second constructor and should be considered semi-default, since the default constructor creates
 an unrealistic gameboard of size 1x1. This constructor creates a gameboard with size row x col.

 @param row
   The number of rows for this gameboard
 @param col
   The number of columns for this gameboard
 */
  public GameBoard(int row, int col)
  {
  	 if (row % 2 == 1)
	 	row--;
	 if (col % 2 == 1)
	 	col--;
		
    numCol = col;
    numRow = row;
    gameBoardMatrix = new int[numRow][numCol];
    clear();
  }

////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 This method clears the gameboard, e.g. sets all positions in the matrix to -1.
 */

  public void clear()
  {
    for (int i = 0; i < numRow; i++)
      for (int j = 0; j < numCol; j++)
        gameBoardMatrix[i][j] = -1;
  }//clear

/*****************************************************************************
   This method clears the row [row], e.g. sets all positions in that row to 0.
   Please note that the row is considered to be a minimum of 1 and a maximum of
   numRow. This is because it's acctually row (row-1) that's erased, since the
   matrix positions are calculated from 0 to numRow-1. For example, let's say that
   you want to erase the first row. If you followed the rules of arrays, you should
   delete row 0. To not confuse the user of the gameboard class you can with this method
   send the parameter 1 if you wanted to erase the first row (instead of 0).

   @param row
     The row to erase.
   @return
     The row that was erased, or -1 if no row was erased
 */
  public int eraseRow(int row)
  {
    if (row <= numRow && row > 0)
    {
      for (int i = 0; i < numCol; i++)
        gameBoardMatrix[row-1][i] = -1;
      return row;
		}

    return -1;
  }//eraseRow

/**
	Erases the specified column, e.g. sets all positions in that column to 0.
	@param col
	  The column to erase
	@return
	  Returns the column that was erased, or -1 if no column was erased
 */
  public int eraseCol(int col)
  {
    if (col <= numCol && col > 0)
    {
      for (int i = 0; i < numRow; i++)
        gameBoardMatrix[i][col-1] = -1;
      return col;
    }
    return -1;
  }//eraseCol

/**
	Erases the specified position, e.g. sets that position to 0.
	@param row
	  Position in row x
	@param col
	  Position in col y
	@return
	 Returns 1 if all went ok, otherwise 0.
 */
  public int erasePos(int row, int col)
  {
    if ((row <= numRow && row > 0) && (col <= numCol && col > 0))
    {
      gameBoardMatrix[row][col] = -1;
			return 1;
    }
		else
			return 0;
  }//erasePos

  public int gameBoardMatrix[][];
  protected int numCol;
  protected int numRow;
}//class