package PuzzleGame;

/**
 A little class for representing the pieces as matrices of ints.
 * <p>Copyright: Copyright Martin Karlsson (c) 2002</p>
 * @author Martin Karlsson
 * @version 1.0
 */
public class PuzzlePiece
{
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
  /*****************************************************************************
   * Default constructor. Creates a piece matrix size 1x1
   */
  public PuzzlePiece()
  {
    numRow = numCol = 1;
    pieceMatrix = new int[numRow][numCol];
  }

 /*****************************************************************************
 * Semi-default constructor. Creates a piece matrix size row x col
 * @param row
 * Number of rows for the piece matrix
 * @param col
 * Number of columns for the piece matrix
 * @param bounds
 * Sets the shape of the piece, see setBounds method for details
 */
  public PuzzlePiece(int row, int col, String bounds)
  {
    numRow = row;
    numCol = col;
    pieceMatrix = new int[numRow][numCol];
    if (setBounds(bounds) == 0)
    {
      String newbounds = new String();
      for(int i = 0; i < (row*col); i++)
        newbounds += "1";
      setBounds(newbounds);
    }
  }
////////////////////////////////////////////////////////////////////////////////

  /*****************************************************************************
   * Sets the bounds/matrix for the piece. For instance, sending the string
   * "101011" to a 3x2 matrix would create the classic L-shaped Tetris-piece,
   * and sending "010111" would create the mirrored L-shaped piece.
   * @param positions
   * A string of numbers, to determine whether a given
   * position in the matrix is true (non-zero number) or false (zero). Beginning from the upper left
   * position of the matrix and moving row-wise downward.
   * @return
   * Return 1 if all went ok, otherwise the method returns 0.
   */

  public int setBounds(String positions)
  {
    if (positions.length() != (numCol*numRow))
      return 0;

    int col = 0;
		int rad = -1;

    for (int i = 0; i < (numRow * numCol); i++)
    {			
      	if ((i % numCol) == 0)
			{
        		rad++;
				col = 0;
			}

			try
			{
				pieceMatrix[rad][col] = Integer.parseInt(positions.substring(i, i+1));
				col++;
			}
			catch (NumberFormatException exc)
			{
				return 0;
			}
    }
		
    return 1;
  }

  /*****************************************************************************
   This method clears the piece matrix, e.g. sets all positions in the matrix to false.
   */

  public void clear()
  {
    for (int i = 0; i < numRow; i++)
      for (int j = 0; j < numCol; j++)
        pieceMatrix[i][j] = 0;
  }//clear
	
	/*****************************************************************************
	Gets the piece's matrix
	@return
		The matrix
	*/
	public int[][] getMatrix()
	{
		return pieceMatrix;
	}
  protected int pieceMatrix[][];
  protected int numRow, numCol;
}