package PuzzleGame;

import java.awt.Image;
import java.awt.Graphics;
import java.awt.Point;

/**
 *
 * <p>Copyright: Copyright Martin Karlsson (c) 2002</p>
 * @author Martin Karlsson, fist_of_heaven@hotmail.com
 * @version 1.0
 */

public class TetrisPuzzlePiece extends PuzzlePiece
{
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
/**
 * See PuzzlePiece class for details about this constructor
 */
  public TetrisPuzzlePiece()
  {
    super();
    moves = true;
		loc = new Point(0, 0);
  }

/**
 * See PuzzlePiece class for details about this constructor
 * @param size
 * Determines the size of the piece' matrix, rows and columns is set to [size] to create a [size]x[size] matrix.
 * @param bounds
 * See PuzzlePiece
 */
  public TetrisPuzzlePiece(int size, String bounds)
  {
    super(size, size, bounds);
    moves = true;
		loc = new Point(0, 0);
  }
////////////////////////////////////////////////////////////////////////////////

/**
 * Draws the piece at it's current location on the board,
 * with the boards offset (x, y) on [page]
 * @param x
 *  The x-position on [page] for the boards upper-left corner
 * @param y
 *  The y-position on [page] for the boards upper-left corner
 * @param page
 *  The Graphics page to draw on
 */
  public void drawPiece(int x, int y, Graphics page)
  {
	 	for(int i = 0; i < numRow; i++)
			for(int j = 0; j < numCol; j++)
				if(pieceMatrix[i][j] != 0)
					page.drawImage(tile, x + (loc.y + j) * tile.getWidth(null), y + (loc.x + i) * tile.getHeight(null), null);
  }

/**
 * Erases the piece at it's current location on the board,
 * with the boards offset (x, y) on [page]
 * @param x
 *  The x-position on [page] for the boards upper-left corner
 * @param y
 *  The y-position on [page] for the boards upper-left corner
 * @param page
 *  The Graphics page to draw on
 */
	public void clearPiece(int x, int y, Graphics page)
	{
		int offsetx = x + loc.y * tile.getWidth(null);
		int offsety = y + loc.x * tile.getHeight(null);

		for (int i = 0; i < numRow; i++)
			for (int j = 0; j < numCol; j++)
				if (pieceMatrix[i][j] != 0)
					page.clearRect(offsetx + j * tile.getWidth(null),
												 offsety + i * tile.getHeight(null),
												 tile.getWidth(null),
												 tile.getHeight(null) );
	}

/**
 * Sets the default tile for this piece
 * @param img
 *   The image that represents the tile
 */
  public void setImage(Image img)
  {
    this.tile = img;
  }

/**
 * Gets the piece's image
 * @return
 * The image
 */
 public Image getImage()
 {
 		return this.tile;
 }

/**
	Rotates the piece right
 */
	public void rotateRight()
	{
		int newMatrix[][] = new int[numCol][numRow];
		boolean emptyRow = true, emptyCol = true;

		for (int row = 0, newcol = numRow-1; row < numRow; row++, newcol--)
			for (int col = 0; col < numCol; col++)
				 newMatrix[col][newcol] = pieceMatrix[row][col];

		for (int i = 0; i < numCol; i++)
		{
			if (emptyRow)
				emptyRow = (newMatrix[0][i] == 0);

			if (emptyCol)
				emptyCol = (newMatrix[i][0] == 0);
		}

		do
		{
			if (emptyRow)
			{
				for (int i = 0; i < numCol; i++)
				{
					for (int j = 0; j < numRow; j++)
						newMatrix[j][i] = newMatrix[j+1][i];

					newMatrix[numRow-1][i] = 0;

					if (emptyRow)
						emptyRow = (newMatrix[0][i] == 0);
				}
			}

			if (emptyCol)
			{
				for (int i = 0; i < numRow; i++)
				{
					for (int j = 0; j < numCol-1; j++)
						newMatrix[i][j] = newMatrix[i][j+1];

					newMatrix[i][numCol-1] = 0;

					if (emptyCol)
						emptyCol = (newMatrix[i][0] == 0);
				}
			}
		}while (emptyRow || emptyCol);


		this.pieceMatrix = newMatrix;
	}

/**
* Returns a copy of the piece by creating a new piece in memory and copy all values
*@return
*  The new piece as a copy of the old
*/
	public TetrisPuzzlePiece getCopy()
	{
		String bounds = new String();

		for (int i = 0; i < numRow; i++)
			for (int j = 0; j < numCol; j++)
				bounds += Integer.toString(pieceMatrix[i][j]);

		TetrisPuzzlePiece newpiece = new TetrisPuzzlePiece(numRow, bounds);
		newpiece.moves = this.moves;
		newpiece.setImage(this.tile);
		newpiece.setLocation(this.getLocation());

		return newpiece;
	}

/**
Returns the current location of the piece
@return
	The current location as a Point-object
*/
	public Point getLocation()
	{
		return loc.getLocation();
	}

/**
Sets the location of the piece, x-axis correspond to columns and y-axis to rows
@param newloc
	The new location
*/
	public void setLocation(Point newloc)
	{
		loc = new Point(newloc);
	}

/**
	A method for determining the piece's matrix right-most non-zero position
        @return
          The right-most position that is non-zero
*/
	public int getRightmostPosition()
	{
		int tempint = 0;

		for (int i = 0; i < numRow; i++)
			for (int j = 0; j < numCol; j++)
				if ( (pieceMatrix[i][j] > 0) && (j > tempint) )
					tempint = j;

		return tempint+loc.y;
	}

/**
	A method for determining the piece's matrix bottom-most non-zero position
        @return
          The bottom-most position that is non-zero
*/
	public int getBottommostPosition()
	{
		int tempint = 0;

		for (int i = 0; i < numRow; i++)
			for (int j = 0; j < numCol; j++)
				if ( (pieceMatrix[i][j] > 0) && (i > tempint) )
					tempint = i;

		return tempint+loc.x;
	}

/**
 * Moves the piece according to given offsets (+/-)
 * @param rowoffset
 	The offset row-wise
 * @param coloffset
 	The offset column-wise
 */
	public void move(int rowoffset, int coloffset)
	{
		loc.translate(rowoffset, coloffset);
	}

  public boolean moves;
	protected Point loc;
  protected Image tile;
}